<?php

/**  // € // to identify files as UTF8
 * handle links defined in linker tool
 *
 * @module          linker_view
 * @author          Wolfgang Studer
 * @copyright       2017 Wolfgang Studer
 * @link            www.LEPTON-cms.org
 * @license         GNU General Public License
 * @license_terms   please see info.php of this module
 * @version         $Id: 1.08.000 | 2017-12-30
 * @lepador_link    http://www.lepton-cms.com/lepador/modules/linker_view.php
 * @forum_link      http://forum.lepton-cms.org/viewtopic.php?f=66&t=3054
*/


// include class.secure.php to protect this file and the whole CMS!
	if ( defined('LEPTON_PATH') == TRUE ):
		include(LEPTON_PATH.'/framework/class.secure.php');
	else:
		$oneback = "../";
		$root = $oneback;
		$level = 1;
		while (( $level < 10 ) && ( file_exists( $root . '/framework/class.secure.php' ) == FALSE )):
			$root .= $oneback;
			$level += 1;
		endwhile;
	
		if ( file_exists($root.'/framework/class.secure.php') == TRUE ):
			include( $root . '/framework/class.secure.php' );
		else:
			trigger_error( sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR );
		endif;
	endif;
// end include class.secure.php


/** ------------------------------------------------------------------------------------
 * run a get_one/all query
 * @return	*		$result-
 * @param	string	$function = __FUNCTION__
 * @param	array	$query = query string
 * @param	string	$kind = 'get_one', 'get_all', 'try_catch'
 ** ------------------------------------------------------------------------------------
 */
if ( function_exists( 'linker_execute_query' ) == FALSE ):
	function linker_execute_query( $function = '', $query = '', $kind = 'get_one', $linker_vars = array() )
	{
		// define globals;
		global $database;
		global $linker_view_debug;

		// overwrite globals with passed vars
		// needed in frontend when some vars are not properly handled by global
		if (( count( $linker_vars ) > 0 )
		and ( array_key_exists( 'linker_view_debug', $linker_vars ) == TRUE )):
			$linker_view_debug = $linker_vars[ 'linker_view_debug' ];
		endif;
		if ( isset( $linker_view_debug ) == FALSE ):
			$linker_view_debug = FALSE;
		endif;

		// do nothing if query is empty
		if ( strlen( $query ) == 0 ):
			return TRUE;
		endif;

		// debug
		if ( $linker_view_debug == TRUE ):
			echo $function . ': ' . $query . '<br />';
		endif;

		// create result & return
		switch ( $kind ):
		/* ********************************************
		 * for SELECT queries with one or multiple selection field.
		 * returns one dimensional array with all values for first rows.
		 * just a pass through option in order to have 1 DB function for all cases.
		 * ********************************************
		 */
		case 'get_one':
			return $database->get_one( $query );
			break;

		/* ********************************************
		 * for SELECT queries with one or multiple selection field.
		 * returns two dimensional array with all values for all rows.
		 * ********************************************
		 */
		case 'get_all':
			$handler = $database->execute_query(
					$query,
					true,
					$result
				);
			return $result;
			break;

		/* ********************************************
		 * for SELECT queries with multiple selection field.
		 * returns one dimensional array with all values for one row.
		 * in case of none or multiple rows as result of select statement, empty array is returned.
		 * ********************************************
		 */
		case 'get_all_one':
			$handler = $database->execute_query(
					$query,
					true,
					$result
				);
			if ( count( $result ) == 1 ):
				return $result[0];
			else:
				return array();
			endif;
			break;

		/* ********************************************
		 * for SELECT queries with 1 selection field.
		 * returns one dimensional array with values of this single field only.
		 * in case of multiple fields in select statement, return is same as "get_all".
		 * ********************************************
		 */
		case 'get_all_single':
			$handler = $database->execute_query(
					$query,
					true,
					$result
				);
			if ( count( $result ) > 0 ):
				$tmpResult = array();
				foreach($result as $item):
					if (( is_array( $item ) == TRUE )
					and ( count( $item )    == 1    )):
						$tmpResult[] = array_values($item)[0];
					else:
						$tmpResult[] = $item;
					endif;
				endforeach;
				return $tmpResult;
			else:
				return array();
			endif;
			break;

		/* ********************************************
		 * for UPDATE, INSERT or DELETE queries.
		 * ********************************************
		 */
		case 'try_catch':
			try	
			{
				$database->query( $query );
				if ( $database->is_error() == TRUE ):
					throw new Exception('Failure during query execution with error: ' . $database->get_error() );
					return FALSE;
				endif;
			}
			catch (Exception $e)
			{
				echo 'Function: ' . $function . '<br />';
				echo 'Query...: ' . $query . '<br />';
				echo '<br />';
				echo 'Caught exception: ' . $e->getMessage() . '<br />';

				return FALSE;
			}
			return TRUE;
			break;

		endswitch;
	}
endif;

?>
