<?php

/**  // € // to identify files as UTF8
 * handle links defined in linker tool
 *
 * @module          linker_view
 * @author          Wolfgang Studer
 * @copyright       2017 Wolfgang Studer
 * @link            www.LEPTON-cms.org
 * @license         GNU General Public License
 * @license_terms   please see info.php of this module
 * @version         $Id: 1.08.000 | 2017-12-30
 * @lepador_link    http://www.lepton-cms.com/lepador/modules/linker_view.php
 * @forum_link      http://forum.lepton-cms.org/viewtopic.php?f=66&t=3054
*/


// include class.secure.php to protect this file and the whole CMS!
	if ( defined('LEPTON_PATH') == TRUE ):
		include(LEPTON_PATH.'/framework/class.secure.php');
	else:
		$oneback = "../";
		$root = $oneback;
		$level = 1;
		while (( $level < 10 ) && ( file_exists( $root . '/framework/class.secure.php' ) == FALSE )):
			$root .= $oneback;
			$level += 1;
		endwhile;
	
		if ( file_exists($root.'/framework/class.secure.php') == TRUE ):
			include( $root . '/framework/class.secure.php' );
		else:
			trigger_error( sprintf("[ <b>%s</b> ] Can't include class.secure.php!", $_SERVER['SCRIPT_NAME']), E_USER_ERROR );
		endif;
	endif;
// end include class.secure.php


/** ------------------------------------------------------------------------------------
 * do input control checks
 * @return	bool		$result-
 * @param	string	$methode = POST, GET
 * @param	array	$array = input array like $_POST
 * @param	array	$fields = list of fields and corresponding filters. Ias also returned with values
 ** ------------------------------------------------------------------------------------
 */
/*
 * Example set of $fields:
 * $fields = array();
 * $fields[ 'field1' ] = array( 'filter' => FILTER_SANITIZE_STRING, 'flags' => FILTER_FLAG_STRIP_HIGH, 'required' => 1 );
 * $fields[ 'array1' ] = array( 'kind' => 'array' );
 * $fields[ 'array1' ][0][ 'field2' ] = array( 'filter' => FILTER_SANITIZE_STRING, 'flags' => FILTER_FLAG_STRIP_HIGH );
 * $fields[ 'array1' ][0][ 'field3' ] = array( 'filter' => FILTER_SANITIZE_STRING );
 * $fields[ 'array1' ][0][ 'field4' ] = array( 'filter' => FILTER_SANITIZE_NUMBER_INT, 'default' = 0,  );
 * $fields[ 'array1' ][0][ 'field5' ] = array( 'filter' => FILTER_SANITIZE_STRING, 'allowed' = array( 'a' => 'aaa', 'b' => 'bbb' ); // checks for key and return value
 * $fields[ 'array1' ][0][ 'field6' ] = array( 'filter' => FILTER_SANITIZE_STRING, 'allowed' = array( 'a', 'b' ); // checks for key only
 * $fields[ 'array2' ] = array( 'kind' => 'array', 'id' => FALSE );
 * $fields[ 'array2' ][0][ 'field2' ] = array( 'filter' => FILTER_SANITIZE_STRING, );
 * $fields[ 'array2' ][0][ 'field3' ] = array( 'filter' => FILTER_SANITIZE_STRING );
 *
 * Notes:
 * 'default' is ignored when 'required => 1
 * 'allowed' array can be "key => value" list or also "key, key" list only
 *
 * Difference between id = FALSE and id = TRUE or ommited:
 * With id = TRUE (or omitted), data array must also have same id key like:
 * 		$data[0]['fieldx'] = value;
 * With id = FALSE, data array has no ID at all like
 * 		$data['fieldx'] = value;
 */
/*
 * Return error:
 *  0 = OK
 * -1 = wrong methode
 * -2 = mandatory field is missing
 * -3 = value not allowed
 * -4 = configuration error
 */

if ( function_exists( 'linker_input_filter' ) == FALSE ):
	function linker_input_filter( $method = '', $array = array(), &$fields = array(), $linker_vars = array() )
	{
		// define globals
		global $MOD_LINKER_VIEW;

		// overwrite globals with passed vars
		// needed in frontend when some vars are not properly handled by global
		if ( count( $linker_vars ) > 0 ):
			$MOD_LINKER_VIEW = $linker_vars[ 'MOD_LINKER_VIEW' ];
		endif;

		// init return
		$error = 0;

		// no input data to check available
		if ( count( $fields ) == 0 ):
			return $error;
		endif;

		// input data source valid
		if ( $_SERVER["REQUEST_METHOD"] != $method ):
			$error = -1;
			$fields[ $error ][ 'error' ] = $MOD_LINKER_VIEW['ERROR'][$error] . $method;
			return $error;
		endif;

		// there is a LEPTON 2.x bug in place which may contains 'amp;' before the key
		$tmp = implode( ';', array_keys( $array ) );
		if ( strpos( $tmp, 'amp;' ) >= 0 ):
			// issue found, we have to do something ...
			$tmp = $array;
			$array = array();
			foreach( $tmp as $tmpkey => $tmpval ):
				if ( strpos( $tmpkey, 'amp;' ) >= 0 ):
					$tmpkey = str_replace( 'amp;', '', $tmpkey );
				endif;
				$array[ $tmpkey ] = $tmpval;
			endforeach;
			unset( $tmp );
		endif;

		// do validation
		$error = linker_filter_detail( $method, $array, $fields, $errors );

		// add errors on root level only
		if ( count( $errors ) > 0 ):
			foreach ( $errors as $errorkey => $erroritem ):
				$fields[ $errorkey ] = $erroritem;
			endforeach;
		endif;

		// return
		return $error;
	}
endif;


if ( function_exists( 'linker_filter_detail' ) == FALSE ):
	function linker_filter_detail( $method = '', $array = array(), &$fields = array(), &$errors = array() )
	{
		// define globals
		global $MOD_LINKER_VIEW;

		// init return
		$error = 0;

		// loop over field array and set filtered value
		foreach( $fields as $field => &$values ):
			// check for required: element is available
			if (( array_key_exists( 'required', $values )	== TRUE )
			and ( $values[ 'required' ]						== 1    )):
				if ( array_key_exists( $field, $array )	== FALSE ):
					$error = -2;
					$errors[ $error ][ 'error' ] = $MOD_LINKER_VIEW['ERROR'][$error] . $field;
					break;
				endif;
			endif;

			// check for array
			if ( array_key_exists( 'kind', $values )	== TRUE ):
				// get kind and id
				$kind = $values[ 'kind' ];
				$id = TRUE;
				if (( array_key_exists( 'id', $values ) == TRUE  )
				and ( $values[ 'id' ]					== FALSE )):
					$id = FALSE;
				endif;
				unset( $values[ 'kind' ] );
				unset( $values[ 'id' ] );

				// do action
				switch ( $kind ):
				case 'array':
					// is field available in array
					foreach ( $fields[ $field ] as $id => &$fieldsitem ):
						if ( $id == TRUE ):
							$data = $array[ $field ][ $id ];
						else:
							$data = $array[ $field ];
						endif;
						$error = linker_filter_detail( $method, $data, $fieldsitem, $errors );
						if ( $error < 0 ):
							break;	// array loop
						endif;
					endforeach;
					break;	// break switch

				default:
					// undefined request
					$error = -4;
					$errors[ $error ][ 'error' ] = $MOD_MEDIA_PRESET['ERROR'][$error] . 'kind=' . $kind;
					break;	// break switch
				endswitch;

				// do error handling
				if ( $error < 0 ):
					break;	// stop validation
				endif;
				continue;
			endif;

			// validate the field
			$tmp = '';
			if ( array_key_exists( $field, $array ) == TRUE ):
				$flags = '';
				if ( array_key_exists( 'flags', $values )	== TRUE ):
					$flags = $values[ 'flags' ];
				endif;
				$tmp = filter_var( trim( $array[ $field ] ), $values[ 'filter' ], $flags );
			endif;

			// set default if exists
			if (( array_key_exists( 'default', $values )	== TRUE )
			and ( strlen( trim( $tmp ))						== 0    )):
				$tmp = $values[ 'default' ];
			endif;

			// check for required: element has content
			if (( array_key_exists( 'required', $values )	== TRUE )
			and ( $values[ 'required' ]						== 1    )):
				if ( strlen( trim( $tmp )) == 0 ):
					$error = -2;
					$errors[ $error ][ 'error' ] = $MOD_LINKER_VIEW['ERROR'][$error] . $field;
					break;
				endif;
			else:
				$values[ 'required' ] = 0;
			endif;

			// check for allowed values
			// if exists return representation of allowed array
			if ( array_key_exists( 'allowed', $values )	== TRUE ):
				if ( strlen( trim( $tmp )) == 0 ):
					// when empty no need to crosscheck against allowed values
				elseif ( array_key_exists( $tmp, $values[ 'allowed' ] )	== TRUE ):
					// return also key
					$values[ 'valuekey' ] = $tmp;
					// key in key=>value assignment found
					$tmp = $values[ 'allowed' ][ $tmp ];
				elseif ( in_array( $tmp, $values[ 'allowed' ] )	== TRUE ):
					// leave tmp as is as it is just a key list
				else:
					// current value not found in allowed values
					$error = -3;
					$errors[ $error ][ 'error' ] = $MOD_LINKER_VIEW['ERROR'][$error] . $field . ' = ' .$tmp;
					break;
				endif;
			endif;

			// save the value
			$values[ 'value' ] = $tmp;
		endforeach;

		// return
		return $error;
	}
endif;

?>
